chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    if (request.type === 'locateRequest') {
      let { x, y} = request.bbox;

      const oldContainer = document.getElementById('walkthrough-spotlight-container');
      if (oldContainer) {
        oldContainer.remove();
      }

      const elem = document.elementFromPoint(x, y );
    
      const box = getElementCoordinates(elem);

      x = box.x;
      y = box.y;
      const width = box.width;
      const height = box.height;
  
      // 1) CREATE/REUSE THE CONTAINER DIV
      //    This will hold our SVG and sit above the page.
      let container = document.querySelector('#walkthrough-spotlight-container');
      if (!container) {
        container = document.createElement('div');
        container.id = 'walkthrough-spotlight-container';
        Object.assign(container.style, {
          position: 'fixed',
          top: '0',
          left: '0',
          width: '100%',
          height: '100%',
          zIndex: '9999',
          pointerEvents: 'auto',
        });
        document.body.appendChild(container);
      }
  
      // 2) CREATE/REUSE THE SVG
      //    This SVG covers the full screen and will have the shading + mask.
      let svg = container.querySelector('#walkthrough-spotlight-bg');
      if (!svg) {
        // Must create SVG elements with createElementNS
        svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
        svg.id = 'walkthrough-spotlight-bg';
  
        // Full-screen styles
        Object.assign(svg.style, {
          position: 'fixed',
          top: '0',
          left: '0',
          width: '100%',
          height: '100%',
          pointerEvents: 'auto',
          display: 'block',
        });
  
        container.appendChild(svg);
        container.addEventListener('click', () => {
            container.remove();
          });
      }
  
      // 3) CREATE/REUSE <defs> & <mask>
      let defs = svg.querySelector('defs');
      if (!defs) {
        defs = document.createElementNS('http://www.w3.org/2000/svg', 'defs');
        svg.appendChild(defs);
      }
  
      let mask = defs.querySelector('#walkthrough-spotlight-mask');
      if (!mask) {
        mask = document.createElementNS('http://www.w3.org/2000/svg', 'mask');
        mask.id = 'walkthrough-spotlight-mask';
        // A large enough area to cover typical screen sizes
        mask.setAttribute('x', '0');
        mask.setAttribute('y', '0');
        mask.setAttribute('width', '8192');
        mask.setAttribute('height', '4096');
        defs.appendChild(mask);
  
        // White rect: the "base" (white = visible in a mask)
        const whiteRect = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
        whiteRect.setAttribute('x', '0');
        whiteRect.setAttribute('y', '0');
        whiteRect.setAttribute('width', '8192');
        whiteRect.setAttribute('height', '4096');
        whiteRect.style.fill = 'rgb(255, 255, 255)';
        mask.appendChild(whiteRect);
      }
  
      // 4) CREATE/REUSE THE GROUP THAT DEFINES THE "CUTOUT" (BLACK = TRANSPARENT IN MASK)
      //    We'll apply the animation styles here (zoom + pulse).
      let spotlightGroup = mask.querySelector('#walkthrough-spotlight-group');
      if (!spotlightGroup) {
        spotlightGroup = document.createElementNS('http://www.w3.org/2000/svg', 'g');
        spotlightGroup.id = 'walkthrough-spotlight-group';
  
        // The same animations from your code:
        spotlightGroup.style.animation =
          'zoom 0.25s linear forwards, pulse 1s 0.25s infinite';
        // For the transform to work properly in an SVG <g>, we set transform-box and transform-origin:
        spotlightGroup.style.transformBox = 'fill-box';
        spotlightGroup.style.transformOrigin = 'center';
        spotlightGroup.style.transform = 'scale(1.05)';
  
        mask.appendChild(spotlightGroup);
      }
  
      // Remove any existing black rect(s) so we can add a fresh one
      while (spotlightGroup.firstChild) {
        spotlightGroup.removeChild(spotlightGroup.firstChild);
      }
  
      // 5) CREATE A BLACK RECT TO "CUT OUT" THE BOUNDING BOX
      //    In an SVG mask, black = fully transparent, white = fully visible.
      const blackRect = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
      blackRect.setAttribute('x', x);
      blackRect.setAttribute('y', y);
      blackRect.setAttribute('width', width);
      blackRect.setAttribute('height', height);
      blackRect.setAttribute('rx', 8);
      blackRect.setAttribute('ry', 8);
      blackRect.style.fill = 'rgb(0, 0, 0)';
      blackRect.style.stroke = 'none';
      // If you want a border around the cutout, you'd typically do it as a second shape or some other trick
      // in an SVG. But the user snippet just "cuts out" the area.
      spotlightGroup.appendChild(blackRect);
  
      // 6) CREATE/REUSE THE SEMI-TRANSPARENT RECT THAT FILLS THE SCREEN, USING THE MASK
      let shadingRect = svg.querySelector('#walkthrough-spotlight-shading');
      if (!shadingRect) {
        shadingRect = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
        shadingRect.id = 'walkthrough-spotlight-shading';
        shadingRect.setAttribute('x', '0');
        shadingRect.setAttribute('y', '0');
        shadingRect.setAttribute('width', '100%');
        shadingRect.setAttribute('height', '100%');
        shadingRect.style.fill = 'rgba(0, 0, 0, 0.5)'; // Semi-transparent overlay
        // Apply the mask
        shadingRect.setAttribute('mask', 'url(#walkthrough-spotlight-mask)');
        shadingRect.style.pointerEvents = 'auto';
        svg.appendChild(shadingRect);
      }
  
      // 7) INJECT KEYFRAMES FOR THE ANIMATIONS ONCE
      if (!document.getElementById('highlight-animations')) {
        const styleSheet = document.createElement('style');
        styleSheet.id = 'highlight-animations';
        styleSheet.innerText = `
            @keyframes zoom {
                0%   { transform: scale(1); }
                100% { transform: scale(1.05); }
            }
            @keyframes pulse {
                0%   { transform: scale(1.05); }
                50%  { transform: scale(1.1); }
                100% { transform: scale(1.05); }
            }
            `;

        document.head.appendChild(styleSheet);
      }
  
      sendResponse({ success: true, box : elem });
    }
  });
  

  function getElementCoordinates(element) {
    const rect = element.getBoundingClientRect();
  
    return {
      top: rect.top,
      left: rect.left,
      right: rect.right,
      bottom: rect.bottom,
      width: rect.width,
      height: rect.height,
      x: rect.x,
      y: rect.y
    };
  }