// let BASE_URL = "http://localhost:8000";
let BASE_URL = "https://www.onboardassistant.com";


function getFormattedTime() {
  const now = new Date();
  const options = { hour: 'numeric', minute: 'numeric', hour12: true };
  return now.toLocaleTimeString('en-US', options);
}


document.addEventListener('DOMContentLoaded', () => {

  let activeTab = 'chat';
  let isConnected = true;
  let isAuthenticated = false;

  const chatTabBtn = document.getElementById('chatTabBtn');
  const loginTabBtn = document.getElementById('loginTabBtn');
  const chatContainer = document.getElementById('chatContainer');
  const loginContainer = document.getElementById('loginContainer');
  const messagesContainer = document.getElementById('messagesContainer');
  const chatForm = document.getElementById('chatForm');
  const chatInput = document.getElementById('chatInput');
  const loginForm = document.getElementById('loginForm');
  const emailInput = document.getElementById('emailInput');
  const passwordInput = document.getElementById('passwordInput');
  const statusIcon = document.getElementById('statusIcon');
  const statusText = document.getElementById('statusText');
  const googleSignInButton = document.getElementById('googleSigninBtn');
  const userInfoContainer = document.getElementById('userInfoContainer');
  const userAvatar = document.getElementById('userInitial');
  const userName = document.getElementById('userName');
  const userEmail = document.getElementById('userEmail');
  const signOutButton = document.getElementById('signOutButton');

  function renderStatus() {
      if (isConnected && isAuthenticated) {
          statusIcon.innerHTML = `
              <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="lucide lucide-wifi"><path d="M12 20h.01"/><path d="M2 8.82a15 15 0 0 1 20 0"/><path d="M5 12.859a10 10 0 0 1 14 0"/><path d="M8.5 16.429a5 5 0 0 1 7 0"/></svg>
          `;
          statusIcon.className = 'active';
          statusText.textContent = 'Active';
          statusText.className = 'active';
      } else {
          statusIcon.innerHTML = `
              <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="lucide lucide-wifi-off"><path d="M12 20h.01"/><path d="M8.5 16.429a5 5 0 0 1 7 0"/><path d="M5 12.859a10 10 0 0 1 5.17-2.69"/><path d="M19 12.859a10 10 0 0 0-2.007-1.523"/><path d="M2 8.82a15 15 0 0 1 4.177-2.643"/><path d="M22 8.82a15 15 0 0 0-11.288-3.764"/><path d="m2 2 20 20"/></svg>
          `;
          statusIcon.className = 'offline';
          statusText.textContent = 'Offline';
          statusText.className = 'offline';
      }
  }

  function renderTab() {
      if (activeTab === 'chat') {
          chatContainer.style.display = 'flex';
          loginContainer.style.display = 'none';
          chatTabBtn.classList.add('active');
          loginTabBtn.classList.remove('active');
      } else {
          chatContainer.style.display = 'none';
          loginContainer.style.display = 'flex';
          chatTabBtn.classList.remove('active');
          loginTabBtn.classList.add('active');
      }
  }

  function renderUI() {
      renderStatus();
      renderTab();
  }

  function handleHighlight(selector) {
      console.log('Highlighting element with selector:', selector);
  }

  async function handleChatSubmit(event) {
    event.preventDefault();
    const text = chatInput.value.trim();
    if (!text) return;

    const userBubble = document.createElement('div');
    userBubble.className = 'message-bubble message-user';
    userBubble.innerHTML = `
        <p class="text-sm">${text}</p>
        <span class="timestamp-user">${getFormattedTime()}</span>
    `;
    messagesContainer.appendChild(userBubble);
    messagesContainer.scrollTop = messagesContainer.scrollHeight;
    
    chatInput.value = '';

    const authInfo = await getAuthInfo();
    if (!authInfo.authToken) {
        displayMessage('error','Please sign in first to use the chat functionality.');
        return;
    }
      chrome.runtime.sendMessage({
        type: 'userChat',
        message: text,
        authToken: authInfo.authToken
    });
  }

  function displayMessage(type, text) {
    const bubble = document.createElement('div');
    if (type === 'user') {
        bubble.className = 'message-bubble message-user';
        bubble.innerHTML = `
            <p class="text-sm">${text}</p>
            <span class="timestamp-user">${getFormattedTime()}</span>
        `;
        messagesContainer.appendChild(bubble);
    } else if (type === 'bot') {
        bubble.className = 'message-bubble message-bot';
        const jsonRegex = /```json\n([\s\S]*)\n```/;
        const match = text.match(jsonRegex);
        let jsonArray = [];
        if (match) {
            const jsonString = match[1].trim();
            try {
                jsonArray = JSON.parse(jsonString);
                console.log(jsonArray);
            } catch (error) {
                console.error("Error parsing JSON:", error);
                console.error("String that was tried to parse:", jsonString)
                sendErrorResponse('Failed to get response from agent.');
            }
        } else {
            console.error("Could not find json in the output")
            sendErrorResponse('Failed to get response from agent.');
        }
        const transformedArray = jsonArray.map(item => {
          const newItem = {};
          for (const key in item) {
            if (key.startsWith("action-")) {
              newItem["action"] = item[key];
            } else if (key.startsWith("rationale-")) {
              newItem["rationale"] = item[key];
            } else {
              newItem[key] = item[key];
            }
          }
          return newItem;
        });
        let stepsHTML = '';
        transformedArray.forEach((step, index) => {
          stepsHTML += `
            <div style="display: flex; align-items: flex-start; margin-bottom: 0.5rem;">
              <!-- Step number -->
              <span style="color: #334155; min-width: 1rem;">
                ${index + 1}.
              </span>
              <div>
                <p style="color: #334155;">${step.action}</p>
                <p style="color: #94a3b8;">${step.rationale}</p>
                <button
                  class="locate-button"
                  data-index="${index}">
                  <svg 
                    xmlns="http://www.w3.org/2000/svg"
                    width="14"
                    height="14"
                    viewBox="0 0 24 24"
                    fill="none"
                    stroke="currentColor"
                    stroke-width="2"
                    stroke-linecap="round"
                    stroke-linejoin="round"
                    class="lucide lucide-target"
                  >
                    <circle cx="12" cy="12" r="10" />
                    <circle cx="12" cy="12" r="6" />
                    <circle cx="12" cy="12" r="2" />
                  </svg>
                  Locate on page
                </button>
              </div>
            </div>
          `;

        });
        bubble.innerHTML = `
            <p class="text-sm">${stepsHTML}</p>
            <span class="timestamp-bot">${getFormattedTime()}</span>
        `;

        bubble.querySelectorAll('.locate-button').forEach(button => {
          button.addEventListener('click', (event) => {
              const idx = parseInt(button.dataset.index, 0);
              const step = transformedArray[idx];
              chrome.runtime.sendMessage({
                type: 'locateRequest',
                stepIndex: idx,
                rationale: step.rationale
              });
          });          
        });
        
        messagesContainer.appendChild(bubble);
    } else if (type === 'error') {
        bubble.className = 'message-bubble message-error';
        bubble.innerHTML = `
            <div>
                <p style="color:#334155;">${text}</p>
            </div>
            <span class="timestamp-bot">${getFormattedTime()}</span>
        `;
        messagesContainer.appendChild(bubble);
    } else if (type === 'success') {
        bubble.className = 'message-bubble message-success';
        bubble.innerHTML = `
            <div>
                <p style="color:#334155;">${text}</p>
            </div>
            <span class="timestamp-bot">${getFormattedTime()}</span>
        `;
        messagesContainer.appendChild(bubble);
    }
    messagesContainer.scrollTop = messagesContainer.scrollHeight;
}

function setButtonLoading(button, isLoading, loadingText = 'Loading...') {
    if (isLoading) {
      button.disabled = true;
      button.dataset.originalHTML = button.innerHTML;
      button.classList.add('button-loading');
      
      button.disabled = true;
      button.innerHTML = loadingText; 
    } else {
      button.disabled = false;
      button.classList.remove('button-loading');
      
      button.innerHTML = button.dataset.originalHTML;
    }
  }


async function handleLoginSubmit(event) {
  event.preventDefault();
  const loginButton = document.querySelector('#loginForm button[type="submit"]');
  setButtonLoading(loginButton, true);

  const email = emailInput.value.trim();
  const password = passwordInput.value.trim();

  if (!email || !password) {
      displayMessage('error', 'Email and password are required.');
      return;
  }

  try {
        const requestBody = { 
          "email":email, 
          "password":password
        };
        const response = await fetch(`${BASE_URL}/api/accounts/auth/customer`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(requestBody),
        });
        console.log(response);
        // const data = await response.json();
        const data = await response.json();
        // const data = response.ok ? await response.json() : await response.json();
        if (response.ok) {
            await chrome.storage.local.set({ 'authToken': data.token });
            const tokenParts = data.token.split('.');
            const payload = JSON.parse(atob(tokenParts[1]));
            updateUIForSignedInUser({
                name: payload.name,
                email: payload.email,
                picture: payload.picture || '/images/icon128.png',
            });

            displayMessage('success', 'Successfully logged in.');
        } else {
            displayMessage('error', data.error || 'Login failed. Please try again.');
        }
    } catch (error) {
        console.error('Error during login:', error);
        displayMessage('error', 'An error occurred while logging in.');
    } finally {
        emailInput.value = '';
        passwordInput.value = '';
        setButtonLoading(loginButton, false);
    }
  }

  chatTabBtn.addEventListener('click', () => {
      activeTab = 'chat';
      renderUI();
  });

  loginTabBtn.addEventListener('click', () => {
      activeTab = 'login';
      renderUI();
  });

  async function getAuthInfo() {
      try {
          const result = await chrome.storage.local.get(['authToken']);
          return {
              authToken: result.authToken
          };
      } catch (error) {
          console.error('Error getting auth info:', error);
          return { authToken: null};
      }
  }

  async function fetchUserInfo(token) {
      try {
          const tokenParts = token.split('.');
          const payload = JSON.parse(atob(tokenParts[1]));
          return {
              email: payload.email,
              name: payload.name,
              picture: payload.picture
          };
      } catch (error) {
          console.error('Error fetching user info:', error);
          throw new Error('Failed to decode ID token');
      }
  }

  async function checkAuthState() {
      const authInfo = await getAuthInfo();
      if (authInfo.authToken) {
          try {
              const userInfo = await fetchUserInfo(authInfo.authToken);
              updateUIForSignedInUser(userInfo);
          } catch (error) {
              console.error('Error checking auth state:', error);
              handleSignOut();
              await chrome.storage.local.remove(['authToken', 'userInfo']);
          }
      } else {
          handleSignOut();
      }
      renderStatus();
  }

  function updateUIForSignedInUser(userData) {
    loginForm.style.display = 'none';
    document.querySelector('.google-signin-separator').style.display = 'none';
    googleSignInButton.style.display = 'none';
    userInfoContainer.style.display = 'flex';
    
    userAvatar.src = userData.picture || '/images/icon128.png';
    if (userData.name != "noname"){
      userName.textContent = userData.name;
    }
    userEmail.textContent = userData.email;
    
    isAuthenticated = true;
    renderStatus();
  }

  async function deleteAuthCookie() {
    try {
        // Delete from backend first
        const response = await fetch(`${BASE_URL}/api/accounts/logout`, {
            method: 'POST',
            credentials: 'include',
            headers: {
                'Content-Type': 'application/json'
            }
        });

        // Remove cookies from both localhost and production domain
        const domains = ['localhost', 'learning-assistant-app-0407.uc.r.appspot.com'];
        
        for (const domain of domains) {
            const cookies = await chrome.cookies.getAll({
                domain: domain,
                name: 'access_token'
            });

            for (const cookie of cookies) {
                await chrome.cookies.remove({
                    url: `http${cookie.secure ? 's' : ''}://${cookie.domain}${cookie.path}`,
                    name: cookie.name
                });
            }
        }

        return true;
    } catch (error) {
        console.error('Error deleting cookie:', error);
        return false;
    }
}


  async function handleSignOut() {
      setButtonLoading(signOutButton, true, 'Signing out...');
      try {
          const authInfo = await getAuthInfo();
          if (authInfo.authToken) {
              await chrome.identity.removeCachedAuthToken({ token: authInfo.authToken });
              await deleteAuthCookie();
              await chrome.storage.local.remove(['authToken', 'userInfo']);
          }
          isAuthenticated = false;
          
          // Reset UI
          loginForm.style.display = 'flex';
          document.querySelector('.google-signin-separator').style.display = 'flex';
          googleSignInButton.style.display = 'flex';
          userInfoContainer.style.display = 'none';
          
          renderStatus();
      } catch (error) {
          console.error('Error signing out:', error);
      } finally {
          setButtonLoading(signOutButton, false);
      }
  }

  googleSignInButton.addEventListener('click', async () => {
    setButtonLoading(googleSignInButton, true);
    try {
        const clientId = '1019580684972-pk7pjsagqjo9tdcp8gs32ri069vqr5vb.apps.googleusercontent.com';
        const redirectUri = chrome.identity.getRedirectURL();
        
        const authUrl = new URL('https://accounts.google.com/o/oauth2/v2/auth');
        const params = {
            'client_id': clientId,
            'response_type': 'id_token',
            'redirect_uri': redirectUri,
            'scope': 'openid email profile',
            'nonce': Math.random().toString(36).substring(2, 15),
            'prompt': 'consent'
        };

        Object.keys(params).forEach(key => 
            authUrl.searchParams.append(key, params[key])
        );

        const responseUrl = await new Promise((resolve, reject) => {
            chrome.identity.launchWebAuthFlow({
                url: authUrl.toString(),
                interactive: true
            }, (response) => {
                if (chrome.runtime.lastError) {
                    reject(chrome.runtime.lastError);
                } else {
                    resolve(response);
                }
            });
        });

        const hashParams = new URLSearchParams(new URL(responseUrl).hash.substr(1));
        const idToken = hashParams.get('id_token');
        
        if (!idToken) {
            throw new Error('No ID token received');
        }

        await chrome.storage.local.set({ 
              'authToken': idToken
        });

        const authResponse = await fetch(`${BASE_URL}/api/accounts/auth/customer`, {
            method: 'POST',
            credentials: 'include',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                token: idToken,
                env: 'extension'
            })
        });
        
        if (!authResponse.ok) {
            displayMessage('error', `Sign-in failed: ${authResponse.statusText}`);
            return;
        }

        const tokenParts = idToken.split('.');
        const payload = JSON.parse(atob(tokenParts[1]));
        
        updateUIForSignedInUser({
            name: payload.name,
            email: payload.email,
            picture: payload.picture
        });
        
        displayMessage('success', 'Successfully signed in with Google.');
        
    } catch (error) {
        console.error('Error during Google Sign-In:', error);
        displayMessage('error', `Sign-in failed: ${error.message}`);
    } finally {
        setButtonLoading(googleSignInButton, false);
    }
  });

  signOutButton.addEventListener('click', handleSignOut);
  checkAuthState();


  chatForm.addEventListener('submit', handleChatSubmit);
  loginForm.addEventListener('submit', handleLoginSubmit);


  chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
      if (request.type === 'agentResponse') {
          displayMessage('bot', request.message);
      } else if (request.type === 'error') {
          displayMessage('error', request.message);
      }
  });

  renderUI();
});
